/************************************************************************
**
** @(#)chklist.cpp		08/25/94		Chris Ahlstrom
**
** Edit with tab-stops at 4.
**
** ------------------------
** 73340.26@compuserve.com
** ------------------------
**
**		Manages a generalized dialog box.  Gives easy access to the
** elements needed to have the FACE_TV library code create a dialog
** box like the following:
**
**				 ----------------- Main Title ------------------
**				|												|
**				| Title of List			Page 2 of 3				|
**				| [ ] Item  1's title							|
**				| [ ] Item  2's title	[ ] Use this list		|
**				| [ ] Item  3's title	[ ] Randomize first		|
**				| [ ] Item  4's title	[ ] Display each time	|
**				| [ ] Item  5's title	[ ] One-at-a-time		|
**				| [ ] Item  6's title							|
**				| [ ] Item  7's title							|
**				| [ ] Item  8's title							|
**				| [ ] Item  9's title							|
**				| [ ] Item 10's title							|
**				| [ ] Item 11's title							|
**				| [ ] Item 12's title							|
**				| [ ] Item 13's title							|
**				| [ ] Item 14's title							|
**				| [ ] Item 15's title							|
**				| [ ] Item 16's title							|
**				|												|
**				|			 -------		 -------			|
**				|			|  Ok	|		|Cancel |			|
**				|			 -------		 -------			|
**				|												|
**				 -----------------------------------------------
**
**		The caller passes along the main title, the title of the
** list, information about the paging, and the list of items.
** The FACE_TV
**
*************************************************************************/

#define CHKLIST_cpp

#include "chklist.men"			// external declarations
#include "chklist.h"			// external declarations


/************************************************************************
** CheckListBase()
**
**		The constructor assumes that the caller has set up the following
** items:
**
**		1.	An array of 'maxitem' integers to reflect the usage-status of
**			'maxitem' items of information.
**
**		2.	A corresponding array of unspecified structures that contains
**			all information for each item.  For example, it might
**			consist of an array of 'maxitem' floating-point numbers.
**
**			The default implementation assumes the items consist of an
**			array of string pointers.
**
*************************************************************************/

CheckListBase::CheckListBase
(
	TDeskTop *desk,				// TurboVision desktop
	int checkoptions,			// word containing desired check-list options
	const char *maintitle,		// string to use for main title
	const char *itemtitle,		// string to use for item title
	CheckListBits *status,		// array word encoding status of each item
	void *items,				// pointer to array of items
	int maxitem,				// count of number of items
	int initpage,				// current page in list [default = 1]
	char *optionlabel			// one option label for now
) :
	deskTop			(desk),
	callerStatus	(status),
	callerItems		(items),
	currentOptions	(checkoptions),
	initialPage		(initpage),
	currentPage		(initpage),
	maximumPage		(((maxitem-1)/CHK_PAGE_SIZE)+1),
	itemCount		(maxitem),
	firstItem		(0),
	bitCount		(0),
	optionLabel		(optionlabel)
{
	/********************************************************************
	** Set the unchanging items of the dialog box.  The rest are set
	** in setCurrentPage(), when run() is called.
	*********************************************************************/

	strncpy(statusArea.mainTitle, maintitle, CHK_TITLE_CHARS);
	strncpy(statusArea.itemTitle, itemtitle, CHK_TITLE_CHARS);
}


CheckListBase::~CheckListBase ()
{
}


/************************************************************************
** run()
**
**		Manages a sequence of dialog boxes.  Examines the current
** page of data, and formats it to fill in the dialog box.
**
*************************************************************************/

#define CHKLIST_DIALOG_ERROR	999

int
CheckListBase::run ()
{
	int breakout = 0;				// flag for parsing list
	int response;					// get dialog response

	statusArea.checkOptions	= currentOptions;	// this will change
	statusArea.currentPage	= initialPage;		// this will change
	statusArea.maximumPage	= maximumPage;		// this will never change

	breakout = 0;
	while (!breakout)					// scan until OK or Cancel
	{
		/************************************************************
		** Set up the data to be displayed in dialog.
		*************************************************************/

		(void) setCurrentPage();					// get caller's data

		/************************************************************
		** Load up the new titles and display the dialog box
		*************************************************************/

		response = runDialog();

		/************************************************************
		** Check the response... 999 [CHKLIST_DIALOG_ERROR] means
		** the dialog box failed, so abort.  1 means that the user
		** clicked OK; 0 means that the user click Cancel.
		*************************************************************/

		#define NEXT_PAGE	1
		#define PREV_PAGE	(-1)
		#define GOTO_PAGE	0

		if (response != CHKLIST_DIALOG_ERROR)
		{
			int pagemove = GOTO_PAGE;

			if (response)							// user clicked OK
			{
				if (dialogData.currentPage == maximumPage)
					breakout = 1;
				else
					pagemove = NEXT_PAGE;

				getCurrentPage();					// update caller's data
			}
			else									// user clicked Cancel
			{
				if (dialogData.currentPage == initialPage)
					breakout = 1;
				else
					pagemove = PREV_PAGE;
			}

			switch (pagemove)
			{
			case NEXT_PAGE:

				if (currentPage < maximumPage)
					currentPage++;
				break;

			case PREV_PAGE:

				if (currentPage > 1)
					currentPage--;
				break;

			case GOTO_PAGE:

				if
				(
					(dialogData.currentPage > 1) &&
					(dialogData.currentPage < maximumPage)
				)
					currentPage = dialogData.currentPage;
				break;
			}
		}
		else
			breakout = 1;							// done
	}
	return response;
}


/************************************************************************
** setCurrentPage()
**
**		Gets the status of the current page *from* the caller.  It gets
** the following items:
**
**		1.	CHKLIST_ACTIVE status of the each field.
**
**		2.	CHKLIST_SELECTED status of the each field.
**
**		3.	The current page-number and some other page-related status
**			items.
**
**		4.	A string to use as the title of each field.  This string
**			can be edited in the box.
**
**		At the end of this routine, the member structures 'dialogArea'
** and 'statusArea' are ready to use.
**
*************************************************************************/

int
CheckListBase::setCurrentPage ()
{
	setNewPage();							// new page might be in force

	unsigned gotvalue = 0;					// start with a clean slate
	unsigned mask = 0x01;					// shifty little bugger

	for (int i = 0; i < bitCount; i++, mask <<= 1)
	{
		if (itemIsActive(i))
		{
			statusArea.itemStatus[i] = CHKLIST_ACTIVE;
			setItemString(i);
			if (itemIsChecked(i))						// call virtual routine
			{
				gotvalue |= mask;						// set bit
				statusArea.itemStatus[i] =				// mask in bit
					CHKLIST_ACTIVE | CHKLIST_SELECTED;
			}
		}
		else
		{
			statusArea.itemStatus[i] = CHKLIST_UNUSED;
			strcpy(statusArea.itemString[i], "<inactive>");
		}
	}
	if (bitCount < CHK_PAGE_SIZE)					// unused items to fill in
	{
		for (int j = bitCount; j < CHK_PAGE_SIZE; j++, mask <<= 1)
		{
			if ((firstItem + j) >= itemCount)			
			{
				statusArea.itemStatus[j] = CHKLIST_UNUSABLE;
				strcpy(statusArea.itemString[j], "[cannot use]");
			}
			else
			{
				statusArea.itemStatus[j] = CHKLIST_UNUSED;
				strcpy(statusArea.itemString[j], "<inactive>");
			}
		}
	}
	statusArea.checkOptions		= currentOptions;	// get copy
	statusArea.currentPage		= currentPage;		// adjust to new page
	statusArea.maximumPage		= maximumPage;		// ensure this is constant

	/********************************************************************
	** Fill in the part essential to the particular user-interface.
	** The following is what Borland's Turbo Vision requires.
	*********************************************************************/

	dialogData.itemSelection	= gotvalue;			// TurboVision checkbox int
	dialogData.checkOptions		= currentOptions;	// get copy
	dialogData.currentPage		= currentPage;		// adjust to new page
	dialogData.maximumPage		= maximumPage;		// ensure this is constant

	copyItemLabels(statusArea.itemString, dialogData.itemString);

	return gotvalue;								// return checked-mask
}


/************************************************************************
** getCurrentPage()
**
**		Puts the status of the current page *back to* the caller.
**
**		Note that the caller must explicitly get the new set of
** options values by calling getCheckOptions() [inline function]
** before destroying the CheckListBase object.
**
*************************************************************************/

void
CheckListBase::getCurrentPage ()
{
	unsigned mask = 0x01;					// shifty little bugger

	setNewPage();							// new page might be in force

	copyItemLabels(dialogData.itemString, statusArea.itemString);

	currentOptions = dialogData.checkOptions;
	for (int i = 0; i < bitCount; i++, mask <<= 1)
	{
		if (dialogData.itemSelection & mask)	// user turned on the check
		{
			statusArea.itemStatus[i] =
				CHKLIST_ACTIVE | CHKLIST_SELECTED;
		}
		else
		{
			statusArea.itemStatus[i] = CHKLIST_ACTIVE;
		}

		/************************************************************
		** Although the user may have checked the box, and it is now
		** turned on, getItemString() might decide, based on the
		** value entered, that the item is bogus, and that the
		** value should really be rendered inactive.  Or that the
		** once active item should now be inactive.
		*************************************************************/

		getItemString(i);				// copy value back to caller
	}
}


/************************************************************************
** setNewPage()
**
**		Sets up new values of key variables whenever it is time to
** process a new page of data.
**
**		Cannot be overridden!
**
*************************************************************************/

void
CheckListBase::setNewPage ()
{
	firstItem	= (currentPage-1) * CHK_PAGE_SIZE;	// index, 1st item on page
	bitCount	= itemCount - firstItem;			// number of items on page
	if (bitCount > CHK_PAGE_SIZE)					// we're not on last page
		bitCount = CHK_PAGE_SIZE;
}


/************************************************************************
** copyItemLabels()
**
**		I thought you could assign arrays in C++.  Wrong again,
** weedmunch.
**
*************************************************************************/

void
CheckListBase::copyItemLabels
(
	ItemLabel &source,
	ItemLabel &dest
)
{
	for (int i = 0; i < CHK_PAGE_SIZE; i++)
		for (int j = 0; j < CHK_ITEM_CHARS+1; j++)
			dest[i][j] = source[i][j];
}


/************************************************************************
** itemIsChecked()
**
**		Returns a 1 if the item is active and should be represented
** by a check-box with a check-mark in it.  If the item is not active,
** then it is incapable of being checked.
**
**		This function is meant to be completely overridden, though it
** does implement checks for a simple list of status words in the
** format of the CheckListBits type.
**
**
** itemIsActive()
**
**		Returns a 1 if the item is active, that is, capable of being
** checked and unchecked.  In other words, this is a simple test of
** presence of the item.
**
**		It can be overridden, if the caller wants to implement a different
** method of encoding status.
**
*************************************************************************/

int
CheckListBase::itemIsChecked
(
	int item
)
{
	int status = callerStatus[firstItem+item];		// caller has set status

	if (!itemIsActive(item))				// is item inactive?
		status = CHKLIST_UNUSED;				// yes, so it is disabled

	return (status & CHKLIST_SELECTED) != 0;
}


int
CheckListBase::itemIsActive
(
	int item
)
{
	int status = callerStatus[firstItem+item];		// caller has set status

	return (status & CHKLIST_ACTIVE) != 0;
}


/************************************************************************
** setItemString()
**
**		Gets the item string *from* the caller.  This routine can
** be overridden.  By default, it assumes that the callerItems points to
** an array of string pointers, so that the string can simply be copied.
**
**		You can imagine that someone might want to instead get a
** the item'th floating point number in an array of floating point
** numbers, and convert it to a string.
**
**		The overriding version should also be prepared to convert
** useless entries to a string such as "<inactive>".
**
*************************************************************************/

void
CheckListBase::setItemString
(
	int item
)
{
	char **stringarray = (char **) callerItems;	// get pointer to 1st pointer

	strncpy
	(
		statusArea.itemString[item],
		*(stringarray+firstItem+item),
		CHK_ITEM_CHARS
	);
}


/************************************************************************
** getItemString()
**
**		Gets the item string and reformats it to put it back *to* the
** caller.  This routine also puts back the status field, too.
**
**		This will probably be overridden a lot.  The default here just
** copies the string back to the right string as pointed to by a
** pointer in an array of pointers.  You can imagine that someone might
** want to instead convert the string to a floating point number.
**
**		The overriding version should also be prepared to convert
** a string such as "<inactive>" or "-1" to a useless entry.
**
*************************************************************************/

void
CheckListBase::getItemString
(
	int item
)
{
	char **stringarray = (char **) callerItems;	// get pointer to 1st pointer

	strncpy
	(
		*(stringarray+firstItem+item),
		statusArea.itemString[item],
		CHK_ITEM_CHARS
	);
	callerStatus[firstItem+item] = statusArea.itemStatus[item];
}


/************************************************************************
** runDialog()
**
**		Runs the Turbo Vision dialog box.  It would certainly be
** a lot more limited than it's Windows counterpart.  Still, it does
** a lot:
**
**		1.	Fills in the title of the check-box group for the items
**			to be selected.
**
**		2.	Fills in the labels of each button in the same check-box
**			group with numbers ranging from 1-16, 17-32, etc.,
**			depending on the page in force.
**
**		3.	Fills in the main title for the dialog window.
**
**		4.	Calls the function to create the dialog as layed out.
**
**		5.	Returns the user's response [0=Cancel, 1=OK, 999=error]
**			to the caller.
**
*************************************************************************/

int
CheckListBase::runDialog ()
{
	int response;

	/********************************************************************
	** The following unwieldy expression means "Go to the MainBox,
	** and get its fields-list. Pick the CHKLIST_ITEMS_INDEXth one.
	** Find its label section.  In this section, you will find the
	** string part of the label spec, and treat it as a pointer to a
	** string This gets us access to the title for the item group.
	*********************************************************************/

	chklistWindow.fields[CHKLIST_ITEMS_INDEX].label.string =
		&statusArea.itemTitle[0];

    /********************************************************************
    ** Now, for the same field, find its data section instead of the
    ** label section.  There you find the TextDataSpec spec, and treat
	** it as a pointer to a string pointer, to get access to the list of
	** button-strings.
    *********************************************************************/

    char **buttontext;				// buffer for building strings

    buttontext = chklistWindow.fields[CHKLIST_ITEMS_INDEX].data.spec.strings;

    for (int count = 0; count < CHK_PAGE_SIZE; count++)
    {
		if (buttontext == 0 || *buttontext == 0 || **buttontext == '\0')
			break;

		sprintf
		(
		    *buttontext, ITEM_DIGIT_FORMAT, count+firstItem+1
		);
		buttontext++;				// next string
    }

	/********************************************************************
	** Hook the title we've copied into the static window structure's
	** main title.
	*********************************************************************/

	chklistWindow.box.title = &statusArea.mainTitle[0];

    /********************************************************************
    ** For the options item, find its data section.  There you find the
	** TextDataSpec spec, and treat it as a pointer to a string pointer,
	** to get access to the list of	button-strings.  If the optionLabel
	** pointer is non-zero, we want to change the "Option 1" string
	** to the optionLabel provided.  Later, we will somehow handle up
	** to 5 options-labels.  Just one for now.
    *********************************************************************/

	char optstring[CHK_OPTION_LENGTH+1];

	if (optionLabel)
		(void) strncpy(optstring, optionLabel, CHK_OPTION_LENGTH);
	else
		(void) strncpy(optstring, OPTION_BLANK, CHK_OPTION_LENGTH);

	optstring[CHK_OPTION_LENGTH] = '\0';

	buttontext =
		chklistWindow.fields[CHKLIST_OPTIONS_INDEX].data.spec.strings;
	buttontext += CHK_OPTION_1_INDEX;

	(void) strcpy(*buttontext, optstring);


    /********************************************************************
	** Execute the manufactured dialog.
    *********************************************************************/

	DialogBox *dialog = new DialogBox
	(
		&chklistWindow,							// layout of dialog box
		(GeneralDialogData) &dialogData,		// data we've assembled
		sizeof(CheckListDTA),					// size of this data
		deskTop									// TurboVision desktop
	);

	if (dialog)
	{
		response = dialog->doDialog();			// need to delete?????
		delete dialog;							// yes, else 48-byte leak
	}
	else
		response = CHKLIST_DIALOG_ERROR;

	return response;
}

