/************************************************************************
**
** @(#)math_aux.c		10/22/94		Chris Ahlstrom
**
**		Additional useful math routines.
**
*************************************************************************/

#include <mem.h>
#include <stdlib.h>
#include <time.h>

#include "math_aux.h"


/************************************************************************
** round
** iround
**
**		Takes a real number, and rounds it up to the next digit, then
** returns it as a long or an integer.
**
*************************************************************************/

long
round
(
	double x
)
{
	if (x > 0)
		x += 0.5;
	else if (x < 0)
		x -= 0.5;
	return((long) x);
}

int
iround
(
	double x
)
{
	if (x > 0.0)
		x += 0.5;
	else if (x < 0.0)
		x -= 0.5;
	return((int) x);
}


/************************************************************************
** dtoul()
** ultod()
**
**		These routines convert between double variables and unsigned
** long variables, with a key feature -- the following calls yield the
** original number, without truncation, within the constraints of the
** implicit conversion from double to float:
**
**		double y, x = 10.3;			// x is the original
**		y = ultod(dtoul(x));		// y == x [no truncation]
**
**		Basically, the calls above reduce the precision of x to float,
** then stuff the bytes into a long. Then the bytes are stuffed
** into a float... order should be preserved under any architecture.
** Then the float is expanded to a double.
**
**		These routines are useful for packing data into the lparam of
** a Windows message function, and extracting it later. They might
** have other uses, so I'm including them here.
**
**		From WINDOWS.H: typedef unsigned long DWORD;
**
**		The UL_PACK_SIZE macro is specific to Borland C++ right now;
** that compiler allows sizeof() in the preprocessor. Anyway, the
** routines depend upon float and long being the same number of bytes.
**
*************************************************************************/

#define UL_PACK_SIZE	sizeof(unsigned long)

unsigned long
dtoul
(
	double x
)
{
	float xf;					/* float version of the double			*/
	unsigned long xul;			/* long version to return				*/

	if (x > 3.4e38)				/* need to flag this case somehow		*/
		x = -1.0;

	xf = (float) x;				/* convert to less spacious version		*/
	memcpy
	(
		(void *) &xul,			/* destination							*/
		(void *) &xf,			/* source								*/
		UL_PACK_SIZE			/* number of bytes						*/
	);

	return xul;
}


double
ultod
(
	unsigned long x
)
{
	float xf;					/* float version of the double			*/

	memcpy
	(
		(void *) &xf,			/* destination							*/
		(void *) &x,			/* source								*/
		UL_PACK_SIZE			/* number of bytes						*/
	);
	
	return (double) xf;
}


/************************************************************************
** getdigit()  [only a local routine]
**
**		A recursive routine to convert an integer to a decimal string.
** The digitslot pointer should point to the *end* of the string to
** convert.  Example:
**
**		int number = 103;		// our number
**		char string[5];			// want to convert it to a 5-digit string
**
**		for (i = 0; i < 5; i++)
**			string[i] = ' ';	// fill string with spaces
**
**		getdigit(number, string+(5-1));
**
**		// the result is string = "  103"
**
*************************************************************************/

static void
getdigit
(
	int number,
	char *digitslot
)
{
	int nnext;
	int trunk;
	int digit;

	nnext		= number / 10;			/* lop off rightmost digit		*/
	trunk		= nnext * 10;			/* shift back leftward			*/
	digit		= number - trunk;		/* recover the rightmost digit	*/
	*digitslot	= (int) (digit - '\0');	/* store it						*/

	if (nnext > 0)
		getdigit(nnext, digitslot-1);
}


/************************************************************************
** itoa10()
**
**		Converts an integer to an ASCII string, base 10.  Similar to
** a call to the stdlib.h routine itoa(x, s, 10), except that the call
** also specifies the minimum number of digits to use, including the
** negative sign.
**
*************************************************************************/

char *
itoa10
(
	int number,
	char *string,				/* destination string					*/
	int digits					/* minimum number of digits				*/
)
{
	char *original = string;

	if (digits > 5)
		digits = 5;

	if (number < 0)
	{
		*string++ = '-';			/* add the negative sign			*/
		digits--;					/* count the slot as used			*/
		number *= -1;				/* convert to a positive number		*/
	}
	if (digits)						/* still have more room				*/
	{
		char *tempstr = string + digits - 1;

		while (tempstr >= string)
			*tempstr-- = '0';		/* fill it with 0 characters		*/

		getdigit(number, string+digits-1);
	}
	return original;
}


/************************************************************************
** random_integers()
**
**		Creates a random array of integers with numbers ranging from
** 0 to siz-1.  Returns a null pointer if a problem occurred.
**
**		Be sure to FREE UP the array when finished with it!
**
** From Borland's stdlib.h:
**
**		#define random(num)	(int) ( ((long)rand()*(num)) / (RAND_MAX+1))
**		#define randomize()	srand((unsigned)time(NULL))
**		#define RAND_MAX	0x7FFFU
**
** We no longer use Wygonski's, though it's still good:
**
**		#define getrandom(min,max)
**					((rand() % (int) (((max)+1)-(min)))+(min))
**
*************************************************************************/

static int inited_random_integers = 0;

int *
random_integers
(
	int siz
)
{
	int *rptr = 0;					/* I'm a cynical bastard			*/

	if (siz > 1)					/* if <= 1, getrandom() screws up	*/
	{
		int *iptr;

		/********************************************************************
		** Initialize the random-number generator the first time.
		*********************************************************************/

		if (!inited_random_integers)
		{
			randomize();
			inited_random_integers = 1;
		}

		rptr = (int *) calloc(siz, sizeof(int));
		if (rptr != 0)
		{
			iptr = (int *) calloc(siz, sizeof(int));
			if (iptr != 0)
			{
				int i;
				int temp;

				for (i = 0; i < siz; i++)
					iptr[i] = i;

				for (i = 0; i < siz; i++) 
				{
					temp = rand() % (siz-i);	// 0 <= rand# < (siz-i)
					rptr[i] = iptr[temp];
					iptr[temp] = iptr[siz-i-1];
				}
				free(iptr);
				iptr = 0;
			}
		}
	}
	return rptr;
}

