/************************************************************************
**
** @(#)msrbutt.cpp		11/23/94	Chris Ahlstrom
**
**		This module interfaces C++ with code for handling the mouse;
** the substrate is Borland's Turbo Vision.
**
**		However, we want the keyboard to remain in control, in case
** the mouse user is different from the keyboard user.  See readKey()
** for details.
**
**		Since this class will be a base class for handling all mouse
** code, it must also do certain things common to all mouse classes.
**
**		ALT_MOUSE_RESPONSE and ALT_MOUSE_POINT_RESPONSE are only
** available on systems that have two mouse ports.  It tries to spawn
** and run the "C:\MOUSE\MOUSE" command to swap mice to be used while
** the response device is active.
**
**		At present, this spawning does not work properly (not sure why!).
** So, instead, we provide a batch file MBOOTH.BAT which must be
** run before running the application program [MLAB.BAT switches it
** back].  But then the 2nd mouse is always the mouse used; the primary
** mouse cannot be used again until MLAB.BAT is run.
**
*************************************************************************/

#define MSRBUTT_cpp

#include <process.h>		// spawnl()

#include "msrbutt.h"		// MouseResponseButton class

extern "C"
{
	#include "tmouse.h"		// a few mouse routines [readMouseButtons()]
}



/************************************************************************
** MouseResponseButton constructor
*************************************************************************/

MouseResponseButton::MouseResponseButton
(
	ResponseType buttontype
)
  :
	ResponseButton			(buttontype),		// a base class
	twoButtonsProhibited	(0)
{
	if
	(
		(responseDevice == ALT_MOUSE_RESPONSE)			||
		(responseDevice == ALT_MOUSE_POINT_RESPONSE)
	)
	{
		(void) setPort(2);						// set up Mouse Port 2
		twoButtonsProhibited = 1;				// prevent subject aborts
	}
}


MouseResponseButton::~MouseResponseButton ()
{
	if (responseDevice == ALT_MOUSE_RESPONSE)	// set up Mouse Port 1
	{
		(void) setPort(1);
	}
}


/************************************************************************
** setPort()
**
**		Available to set up the alternate or main mouse port.
** However, we have no method of passing back errors to the caller
** of the class constructor.
**
**		Note that two MOUSE commands must be used:  one to turn off
** the existing driver, and one to turn it back on with the new
** port number.
**
**		Later, I'll try to dig up the proper mouse-driver calls,
** and use them, instead of spawning.
**
**		Note that we have to mouse.com in the right directory, for
** now.
**
*************************************************************************/

int
MouseResponseButton::setPort
(
	int port									// 1==main, 2==alternate
)
{
	char *commandpath = "C:\\MOUSE\\MOUSE";
	char arg1[4];
	char *off = "off";
	int errcode = 0;

	if (port < 1 || port > 2)
		port = 1;

	errcode = spawnl(P_WAIT, commandpath, commandpath, off, 0);

	if (errcode != -1)
	{
		arg1[0] = '/';
		arg1[1] = 'i';
		arg1[2] = (char) port + '0';
		arg1[3] = '\0';

		errcode = spawnl(P_WAIT, commandpath, commandpath, arg1, 0);
	}

	if (errcode == -1)
		responseDevice = MOUSE_RESPONSE;		// don't do in constructor

	return errcode;
}


/************************************************************************
** readKey() override
**
**	Function to read the mouse buttons.
**
**	button is a bitmask; the following values apply:
**
**		  Bit   Meaning if on
**		 -----  --------------------------------------------
**		0 = left button is down (LEFT_MOUSEBUTTON)
**		1 = right button is down (RIGHT_MOUSEBUTTON)
**		2 = center button is down (CENTER_MOUSEBUTTON)
**		3 to 15 are reserved
**
**	See the Response typedef in rbuttons.h.
**
*************************************************************************/

int
MouseResponseButton::readKey ()
{
	int button;
	
	button = ResponseButton::readKey();		// check keyboard first

	if (button == 0)						// nothing from the keyboard
	{
		if (twoButtonsProhibited)
		{
			do
			{
				button = readMouseButtons();

			} while (button == (LEFT_MOUSEBUTTON | RIGHT_MOUSEBUTTON));
		}
		else
			button = readMouseButtons();

		dawdle(button);						// wait until keys are released
	}
	lastResponse = (Response) button;

	return button;
}


/************************************************************************
** waitKey() override
*************************************************************************/

int
MouseResponseButton::waitKey ()
{
	int button;

	while (readMouseButtons() != NO_BUTTON)	// discard previous state
		;									// wait until it goes away

	do
	{
		button = readKey();
	
	} while (button == (int) NO_BUTTON);

	return button;
}



/************************************************************************
** readCoordinates()
**
**		By default, acts exactly like readKey().  However, it is intended
** to leave an opening for a device return a two-dimensional coordinate
** pair.
**
*************************************************************************/

int
MouseResponseButton::readCoordinates
(
	int &xvalue,
	int &yvalue
)
{
	int button;
	int x, y;
	
	button = ResponseButton::readCoordinates	// check keyboard first
	(
		x, y									// tricky old C idea
	);

	if (button == 0)						// nothing from the keyboard
	{
		if (twoButtonsProhibited)
		{
			do
			{
				button = readMouse			// instead of readMouseButtons()
				(
					&x, &y					// ahhh, a plain C routine!
				);

			} while (button == (LEFT_MOUSEBUTTON | RIGHT_MOUSEBUTTON));
		}
		else
			button = readMouse				// instead of readMouseButtons()
			(
				&x, &y						// vanilla C
			);

		dawdle(button);						// wait until keys are released
	}
	lastResponse = (Response) button;
	xvalue = x;
	yvalue = y;

	return button;
}

/************************************************************************
** dawdle()
**
**		If any mouse buttons are down, this function dawdles while either
** of the buttons are still down, to avoid racing to the next function
** while the buttons are still down.
**
*************************************************************************/

void
MouseResponseButton::dawdle
(
	int button
)
{
	if (button & (LEFT_MOUSEBUTTON | RIGHT_MOUSEBUTTON))
	{
		int state;

		do
		{
			state = readMouseButtons();

		} while (state & (LEFT_MOUSEBUTTON | RIGHT_MOUSEBUTTON));
	}
}
