/************************************************************************
**
** @(#)rbox_bgi.cpp		11/14/94		Chris Ahlstrom
**
** C++; edit with tab-stops set to 4.
**
**		This module defines the ResponseBGIInterior class, which
** objectifies the abstract class ResponseBoxInterior for use with
** the Borland BGI library.
**
**		This module has the same overall function as rbox_tvi.cpp/h,
** but instead of managing response lights in a TV window, it creates
** the lights using Borland's BGI graphics interface.
**
**		Unlike ResponseTVInteriior in rbox_tvi.cpp, ResponseBGIInterior
** actually does all the work, since we don't have to deal with any
** Turbo Vision crap.
**
**		Both classes use the same ResponseBoxLight data for their
** boxes however.  Both convert the virtual coordinates (range of
** 0 to 10000) to on-screen coordinates.  While the TV version converts
** to character locations, this one converts to screen pixels.
**
**		The procedure displays roughly the following layout (you're
** screen will differ in details):
**
**
**		 ---------							 ---------
**		|  READY  |							| ANSWER |
**		 ---------							 ---------
**		 ---------							 ---------
**		|		  |							|		 |
**		|		  |							|		 |
**		|		  |							|		 |
**		 ---------							 ---------
**
**				 ----------			 ----------
**				|Interval 1|		|Interval 2|
**				 ----------			 ----------
**				 ----------			 ----------
**				|		   |		|		   |
**				|		   |		|		   |
**				|		   |		|		   |
**				 ----------			 ----------
**
**
**				 ----------			 ----------
**				|Response 1|		|Response 2|
**				 ----------			 ----------
**				 ----------			 ----------
**				|		   |		|		   |
**				|		   |		|		   |
**				|		   |		|		   |
**				 ----------			 ----------
**
**
**		See rbox_int.cpp/h for an explanation about the representation
** of these lights.
**
*************************************************************************/

#define RBOX_BGI_cpp

#include <stdio.h>
#include <string.h>
#include <graphics.h>

#include "rbox_bgi.h"			// ResponseBGIInterior class and ancestors


/************************************************************************
** ResponseBGIInterior constructor and destructor.
**
**		Note that, since we allocate a semi-permanent finalBox, it
** might prove wise to add an assignment operator and copy constructor.
**
**		resplightParameters[] and resplightCount[] are defined in
** respvga.h -- they implement a somewhat virtual (flexible) scheme
** of specifying the response-interior lights.
**
*************************************************************************/

ResponseBGIInterior::ResponseBGIInterior
(
	ResponseBoxLight resplightparameters[],
	int resplightcount,
	ResponseBoxText resptextparameters[],
	int resptextcount,
	unsigned *bmasklist,
	int alllightcount,
	const Rectangle &bounds,					// size of the "window"
	const char *atitle,							// optional title
	ScreenCombo combo
) :
	ResponseBoxInterior	(),
	graphBox			(&resplightparameters[0]),
	finalBox			(new ResponseBoxLight[resplightcount]),
	graphTextBox		(&resptextparameters[0]),
	finalTextBox		(new ResponseBoxText[resptextcount]),
	bgiControl			(new BGIManager(combo))
{
	lightCount = alllightcount;
	bitMaskList = new unsigned[lightCount];
	if (bitMaskList)
	{
		for (int i = 0; i < lightCount; i++)
			bitMaskList[i] = bmasklist[i];		// link to external bit list
	}

	if (bgiControl && bgiControl->graphicsOK())
	{
		graphicsReady = 1;						// is this the best spot?
		boxAspectConvert(bounds, atitle);					// convert coords.
		setbkcolor(finalBox[BOX_BACKGROUND_LIGHT].color);	// set color

		/****************************************************************
		** Set up rest of LIGHT bits.  The call is to the default
		** startState() in rbox_int.cpp/h.
		*****************************************************************/

		startState();							// add usual stuff to bits
	}
}


/************************************************************************
** ResponseBGIInterior destructor
**
**		I'm wondering if it's safe to put the responseOff(1) call
** inside the ResponseInterior destructor instead.  I don't remember
** the order of destructor calls in inheritance.
**
*************************************************************************/

ResponseBGIInterior::~ResponseBGIInterior ()
{
	responseOff(1);					// turn off all lights
	if (bitMaskList)
		delete [] bitMaskList;
	if (bgiControl)
		delete bgiControl;
	if (finalTextBox)
		delete [] finalTextBox;
	if (finalBox)
		delete [] finalBox;
}


/************************************************************************
** virtual int startState() not overridden
*************************************************************************/

/************************************************************************
** virtual int drawGraphic()
** virtual int drawText()
** virtual int drawSpecial()
*************************************************************************/

int
ResponseBGIInterior::drawGraphic
(
	unsigned lpboffset,
	unsigned onstate
) 
{
	if (onstate)
		makeLightOn((int) lpboffset);
	else
		makeLightOff((int) lpboffset);

	return 0;
}


int
ResponseBGIInterior::drawText
(
	unsigned lpboffset,
	unsigned onstate
) 
{
	settextstyle
	(
		finalTextBox[lpboffset].font,
		finalTextBox[lpboffset].direction,
		finalTextBox[lpboffset].charsize
	);
	if (finalTextBox[lpboffset].textstring != 0)
	{
		if (onstate)
			setcolor(finalBox[BOX_INTERVAL_1].color);
		else
			setcolor(finalBox[BOX_BACKGROUND_LIGHT].color);
		outtextxy
		(
			finalTextBox[lpboffset].x0,
			finalTextBox[lpboffset].y0,
			finalTextBox[lpboffset].textstring
		);
	}
	else
	{
		char lineout[80];

		sprintf
		(
			lineout, "?Missing string: text light %d", (int) lpboffset
		);
		outtextxy(0, 0, lineout);
	}
	finalTextBox[lpboffset].status = onstate != 0;;

	return 0;
}


int
ResponseBGIInterior::drawSpecial
(
	int 		/* lightnumber*/ ,
	unsigned	/* onstate	  */
)
{
	return 0;
}



/************************************************************************
** drawThis()
*************************************************************************/

int
ResponseBGIInterior::drawThis ()
{
	return ResponseBoxInterior::drawThis();	// set all lights
}


/************************************************************************
** boxAspectConvert()
**
**		Generates the proper coordinates to specify the layout
** of a response box, given the type of response box settings in use
** for the current graphics monitor.
**
**		In addition, the coordinates are scaled to fit inside a box
** of a particular size (in units of lines and characters).
**
**		This routine is called in the draw() routine, which is overridden
** here. By calling it there, the size of the lights will somewhat
** follow the size of the box, if the user resizes it.
**
*************************************************************************/

void
ResponseBGIInterior::boxAspectConvert
(
	const Rectangle &wbox,			// window box into which to fit it all
	const char *atitle				// optional title
)
{
	if (bgiControl == 0)
		return;

	for (int lite = 0; lite < lightCount; lite++)
	{
		unsigned lpoff = ((bitMaskList[lite]&LIGHT_PARAMETERS) >> 8) & 0x00FF;

		if (bitMaskList[lite] & LIGHT_IS_TEXT)
			bgiControl->textAspectConvert
			(
				wbox, graphTextBox[lpoff], finalTextBox[lpoff]
			);
		else if (bitMaskList[lite] & LIGHT_IS_SPECIAL)
			bgiControl->lightAspectConvert
			(
				wbox, graphBox[lpoff], finalBox[lpoff]
			);												//BOGUS
		else
			bgiControl->lightAspectConvert
			(
				wbox, graphBox[lpoff], finalBox[lpoff]
			);

		if (bgiControl->monitorType() != BGI_USE_MAIN_VGA)
		{
			if (bitMaskList[lite] & LIGHT_IS_BACKGROUND)
				finalBox[lpoff].color = MONO_BACKGROUND_COLOR;
			else if (!(bitMaskList[lite] & (LIGHT_IS_TEXT|LIGHT_IS_SPECIAL)))
				finalBox[lpoff].color = MONO_VISIBLE_COLOR;
		}
		if (lite == BOX_BANNER_LIGHT && (atitle != 0))
			finalTextBox[lpoff].textstring = atitle;
	}
}


/************************************************************************
** makeLightOn()
** makeLightOff()
**
**		Turns on or off a single light.  Since drawing of the lights
** can be redone at any time, we must always have the current status
** of lights available. Hence, these two functions are very simple...
** they merely set or reset the state flag for the light.
**
**		Note that the finalBox[lightnumber] coordinates already contain
** the converted coordinates of the light, as calculated by
** boxAspectConvert().
**
*************************************************************************/

void
ResponseBGIInterior::makeLightOn
(
	int lightnumber
)
{
	if (readyForUse)
	{
		setfillstyle
		(
			finalBox[lightnumber].fillstyle,
			finalBox[lightnumber].color
		);
		bar
		(
			finalBox[lightnumber].x0,
			finalBox[lightnumber].y0,
			finalBox[lightnumber].x1,
			finalBox[lightnumber].y1
		);
	}
	finalBox[lightnumber].status = 1;
}


void
ResponseBGIInterior::makeLightOff
(
	int lightnumber
)
{
	if (readyForUse)
	{
		setfillstyle
		(
			finalBox[BOX_BACKGROUND_LIGHT].fillstyle,
			finalBox[BOX_BACKGROUND_LIGHT].color
		);
		bar
		(
			finalBox[lightnumber].x0,
			finalBox[lightnumber].y0,
			finalBox[lightnumber].x1,
			finalBox[lightnumber].y1
		);
	}
	finalBox[lightnumber].status = 0;
}


