/************************************************************************
**
** @(#)rbox_dbg.cpp		11/11/94		Chris Ahlstrom
**
** C++; edit with tab-stops at 4.
**
**		This module defines the base class ResponseBGIDevice, which
** should be derived from a common ancestor of ResponseBoxDevice,
** but is not at present.  It is derived from ResponseBoxDeviceBase,
** which will eventually be made an ancestor of ResponseBoxDevice, too.
**
**		The class works by creating a response-window object for
** presenting visual feedback, and a button object for accepting
** responses from the user.
**
**		This module supercedes the RESPWIND module.
**
**		This module is an alternate version of RBOX_DEV; that one
** works with Turbo Vision windows, this one works with a separate
** graphics module.  We will unify them all someday.
**
*************************************************************************/

#define RBOX_DBG_cpp

#include <stdio.h>
#include <conio.h>
#include <ctype.h>

#include "mkbutton.h"			// ResponseButton creation class
#include "rbox_dbg.h"			// ResponseBGIDevice class


/************************************************************************
** Special lights
**
**		We allow the Ready and Answer lights to be either colored
** squares or text strings.
**
*************************************************************************/

#define NORM_READY_LIGHT	(LIGHT_IS_TOGGLE|0x0200)
#define NORM_ANSWER_LIGHT	(LIGHT_IS_TOGGLE|0x0300)

#define TEXT_READY_LIGHT	(LIGHT_IS_TOGGLE|0x0400|LIGHT_IS_TEXT)
#define TEXT_ANSWER_LIGHT	(LIGHT_IS_TOGGLE|0x0500|LIGHT_IS_TEXT)


/************************************************************************
** ResponseBGIDevice
**
**		I'm not sure how to subdivide the work.  The constructor here
** fixes up an array of lights to fit the desired features.  We could
** have implemented the different features as derived objects, but
** I'm not sure it's important here, since the differences in features
** are mainly in the number of lights to be shown.
**
**		Can't make copies and such until we write the copy constructor
** and assignment operator.
**
*************************************************************************/

ResponseBGIDevice::ResponseBGIDevice
(
	ResponseType device,
	const Rectangle &bounds,
	const char *atitle,			// later, could write at top of screen
	ScreenCombo combo,			// mouse/monitor combination
	ResponseFeature style,		// response-box style
	int intervalcount			// number of intervals (1 to 3)
) :
	ResponseBoxDeviceBase
	(
		device, bounds, atitle, combo, style, intervalcount
	)
{
	afcBitMaskList[BOX_READY_LIGHT]		= NORM_READY_LIGHT;
	afcBitMaskList[BOX_ANSWER_LIGHT]	= NORM_ANSWER_LIGHT;

	setupBoxLights(intervalcount);

	switch (style)
	{
	case RF_STANDARD:

		resplightParameters[BOX_RESPONSE_1].color =
		resplightParameters[BOX_RESPONSE_2].color =
		resplightParameters[BOX_RESPONSE_3].color =
			resplightParameters[BOX_INTERVAL_2].color;
		break;

	case RF_PLAIN:

		resplightParameters[BOX_RESPONSE_1].color =
		resplightParameters[BOX_RESPONSE_2].color =
		resplightParameters[BOX_RESPONSE_3].color =
			resplightParameters[BOX_BACKGROUND_LIGHT].color;
		break;

	case RF_CYAN:

		resplightParameters[BOX_RESPONSE_1].color =
		resplightParameters[BOX_RESPONSE_2].color =
		resplightParameters[BOX_RESPONSE_3].color =
			CYAN;
		break;

	case RF_TEXT_WARNING:

		afcBitMaskList[BOX_READY_LIGHT]		= TEXT_READY_LIGHT;
		afcBitMaskList[BOX_ANSWER_LIGHT]	= TEXT_ANSWER_LIGHT;
		break;
	}

	subInterior = new ResponseBGIInterior
	(
		resplightParameters, resplightCount,
		resptextParameters, resptextCount,
		afcBitMaskList, allLightCount,
		bounds, atitle, combo
	);
}


/************************************************************************
** ~ResponseBGIDevice
*************************************************************************/

ResponseBGIDevice::~ResponseBGIDevice()
{
}


/************************************************************************
** setupBoxLights()
**
**		Sets up X values for the response and interval lights, so that
** they will work for 1, 2, or 3 intervals.  We use a coarse scale of
** 1/8ths as the unit of measure for offset and size of the lights,
** and we map the points onto the virtual scale (0 to 10000).
**
**		 _______________________________
**		|								|
**		|		allowable X range		|
**		|_______________________________|
**
**		0	.	.	.	.	.	.	.	1.0			unity scale
**
**	LEFTX0 = 1560					RGHTX1 = 8440	10000 (virtual) scale
**
**				 _______________
**				|				|
**				|				|					1 interval
**				|_______________|
**
**				2/8				6/8
**
**
**		 ___________		 ___________
**		|			|		|			|
**		|			|		|			|			2 intervals
**		|___________|		|___________|
**
**		0			3/8		5/8			8/8
**
**
**		 _______	 _______	 _______
**		|		|	|		|	|		|
**		|		|	|		|	|		|			3 intervals
**		|_______|	|_______|	|_______|
**		0	   2/8 3/8	   5/8 6/8		8/8
**
**
**		The Y values never change in this implementation.  They are
** as set up in rbox_afc.h and rbox2afc.hdt.
**
**		If the interval count is < 0, then we use the 2-interval
** stuff hardwired in.
**
*************************************************************************/

void
ResponseBGIDevice::setupBoxLights
(
	int intervalcount
)
{
	if (intervalcount > 0)
	{
		for (int j = 0; j < 3; j++)
		{
			/************************************************************
			** Disable all the response and interval lights at first.
			*************************************************************/

			afcBitMaskList[BOX_INTERVAL_1+j] &= ~LIGHT_IS_TOGGLE;
			afcBitMaskList[BOX_RESPONSE_1+j] &= ~LIGHT_IS_TOGGLE;
		}

		double size8ths = 5.0 - intervalcount;	// x size of boxes in 8ths
		double oinc8ths = 5.0;					// x offset increment
		double strt8ths = 0.0;					// x offset at start

		if (intervalcount > 3)
			intervalcount = 2;

		if (intervalcount == 3)
			oinc8ths = 3.0;						// x offset increment

		if (intervalcount == 1)
			strt8ths = 2.0;						// x offset at start

		for (int i = 0; i < intervalcount; i++, strt8ths += oinc8ths)
		{
			/************************************************************
			** Enable the chosen response and interval lights.  Then
			** modify their X location and X size appropriately.
			*************************************************************/

			afcBitMaskList[BOX_INTERVAL_1+i] |= LIGHT_IS_TOGGLE;
			afcBitMaskList[BOX_RESPONSE_1+i] |= LIGHT_IS_TOGGLE;

			resplightParameters[BOX_INTERVAL_1+i].x0 =
			resplightParameters[BOX_RESPONSE_1+i].x0 =
				xLocation(strt8ths);

			resplightParameters[BOX_INTERVAL_1+i].x1 =
			resplightParameters[BOX_RESPONSE_1+i].x1 =
				xLocation(strt8ths + size8ths);
		}
	}
}


int
ResponseBGIDevice::xLocation
(
	double eighths
)
{
	double xmain0	= (double) LEFTX0;
	double xmain1	= (double) RGHTX1;
	double xrange	= xmain1 - xmain0;

	xmain1 = xmain0 + (eighths * 0.125) * xrange + 0.5;

	return (int) xmain1;
}
