/************************************************************************
**
** @(#)rbox_inb.h		11/17/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
** C++; edit with tab-stops = 4.
**
**		Defines the ResponseBoxInteriorBase class, which supercedes the
** ResponseInterior class of the RESPINTR modules.  This class is
** solely a base class, quite independent of video support.  It is
** an ABC (abstract base class).
**
**		The derived classes define what is drawn, and how it is drawn.
** See rbox_inb.cpp for a discussion of how things can be drawn.
**
*************************************************************************/

#if !defined(RBOX_INB_h)								// { RBOX_INB_h
#define RBOX_INB_h


/************************************************************************
** Values for conditionally calling allOff()
**
**		Some uses of these otherwise straightforward value constitute
** TRICKY CODE.
**
**	TURN_OFF
**
**		This value is used for response-boxes that need a more
**		standard handling of lights.  Recall that we implemented
**		the 1AFC, 2AFC, and 3AFC response boxes toggling on-bits
**		for each light.  Unless we specifically turned an on-bit
**		off, that light would stay on, because it would be redrawn
**		as on when the whole screen was redrawn.  But, in the
**		speech-token response box, we do not store the on-states,
**		so we must manipulate each light-item separately.  This means
**		we need a way to have a light's function turn its light off.
**		Hence we need the TURN_OFF parameter value.
**
**	KEEP_THEM_ON
**
**		This value indicates to /not/ turn off the other lights.
**		In some classes, the allOff() routine might not do this, anyway.
**
**	TURN_THEM_OFF
**
**		This value indicates to turn off the other lights.
**
*************************************************************************/

#define TURN_OFF			(-1)	// turn only this light off
#define KEEP_THEM_ON		  0		// keep the other lights on
#define TURN_THEM_OFF		  1		// turn the other lights off

#define LIGHT_OFF			  0		// turn the light off pronto
#define LIGHT_ON			  1		// turn the light on pronto


/************************************************************************
** ResponseBoxInteriorBase
*************************************************************************/

class ResponseBoxInteriorBase
{

public:

	ResponseBoxInteriorBase();				// constructor

	virtual ~ResponseBoxInteriorBase ();	// destructor

	virtual int drawThis () = 0;			// redraws whole interior
	virtual int startState () = 0;			// turn on all the lights
	virtual int intervalLight
	(
		int interval,
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int responseLight
	(
		int response,
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int responseOff					// turn off all but a couple
	(
		int allofthem = 0					// turn those off, too
	) = 0;
	virtual int readyLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int failureLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int bannerLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int answerLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int wrongLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int rightLight
	(
		int offem = TURN_THEM_OFF
	) = 0;
	virtual int feedbackLight
	(
		int interval,
		int response
	) = 0;

public:										// just in case

	virtual void cursorOn					// activate our own cursor
	(
		double xfraction = -1.0,			// 0 to 1, relative start of cursor
		double yfraction = -1.0				// 0 to 1, relative start of cursor
	);
	virtual void cursorOff ();				// deactivate our own cursor
	virtual void cursorMove					// move cursor to new place
	(
		int x,
		int y
	);
	virtual int getBoxNumber
	(
		int x,
		int y
	);

protected:									// internal support

	virtual int drawSpecial					// redraw special light
	(
		int lightnumber,
		unsigned onstate
	) = 0;
	virtual int drawText
	(
		unsigned lpboffset,
		unsigned onstate
	) = 0;
	virtual int drawGraphic
	(
		unsigned lpboffset,
		unsigned onstate
	) = 0;
	virtual void allOff () = 0;				// utility function

	int lightCount;							// max number lights supported
	int readyForUse;						// can't draw until ready
	int graphicsReady;						// can't draw if no graphics
	int noMoreBanner;						// only want banner at start

};


#endif													// } RBOX_INB_h
