/************************************************************************
**
** @(#)rbox_int.cpp		08/25/94		Chris Ahlstrom
**
** C++ version, requires Turbo Vision by Borland
**
**		This module defines the ResponseBoxInterior class, which defines
** a standard set of calls that the ResponseBoxDevice class can count
** on in handling user-interface I/O.
**
**		The model for the interior is simple:
**
**		1.	The interior's state is represented by a list of bit-masks,
**			one for each light.  The lighting functions operate by
**			setting the bits appropriate for each light in the purview
**			of the lighting function.  Then each lighting function
**			calls the drawThis() function, which reviews the list to
**			see what drawing needs to be done.
**
**		2.	The interior has a starting state, set up by the constructor
**			and by the startState() function.  Normally, it causes all
**			the lights in the interior to be on.  This lets the subject
**			or user know that the equipment is all set up and ready to
**			go.
**
**		3.	The interior has a number of "lit" states, representing
**			combinations of up to 64 lights.  Each light is represented
**			by a bit-mask that tells what kind of light it is, whether
**			it is already on or off, whether it can be toggled,
**			or whatever the light needs.
**
**			See the LightState enum for the bit-meanings.
**
*************************************************************************/

#define RBOX_INT_cpp

#include <conio.h>			// cputs()

#include "rbox_int.h"		// ResponseBoxInterior class

const int ResponseBoxInterior::lightCountMax = LIGHT_COUNT_MAX;


/************************************************************************
** ResponseBoxInterior constructor and destructor.
**
**		Note that, since we allocate a semi-permanent textBox, it
** might prove wise to add an assignment operator and copy constructor.
**
*************************************************************************/

ResponseBoxInterior::ResponseBoxInterior ()
 :
	ResponseBoxInteriorBase(),			// very basics class
	bitMaskList		(0),				// derived class's bitMaskList
	lightCount		(0),				// max number lights in class
	readyForUse		(0),				// flag raised later
	graphicsReady	(0),				// flag must be raised later
	noMoreBanner	(0)					// flag raised later
{
	//////////////////////////////////////////////////////////////////////
	// Do not call startState() here!  Call it in the derived class!    //
	//////////////////////////////////////////////////////////////////////
}


ResponseBoxInterior::~ResponseBoxInterior ()
{
}


/************************************************************************
** allOff()
**
**		A handy-dandy utility.  Turns off all the necessary LIGHT_IS_ON
** bits.  Then makes sure that the background and frame lights will
** be lit.  Also the banner light.
**
*************************************************************************/

void
ResponseBoxInterior::allOff ()
{
	if (bitMaskList)
	{
		for (int i = 0; i < lightCount; i++)
			bitMaskList[i] &= ~LIGHT_IS_ON;

		bitMaskList[BOX_BACKGROUND_LIGHT]	|= LIGHT_IS_ON;
		bitMaskList[BOX_FRAME_LIGHT]		|= LIGHT_IS_ON;

		if (!noMoreBanner)
			bitMaskList[BOX_BANNER_LIGHT]	|= LIGHT_IS_ON;
	}
}


/************************************************************************
** virtual int startState()
**
**		This function sets up all the lights, normally making them
** all visible, making it clear to the user that things are ready to
** go.  If some hardware problem occurs, a non-zero error code is
** returned.
**
**		Note that we can't call this function in the constructor
** because lightCount and bitMaskList won't have good values yet.
**
*************************************************************************/

int
ResponseBoxInterior::startState ()
{
	int errcode = 0;

	if (bitMaskList)
	{
		noMoreBanner = 0;							// allow banner (again)

		for (int j = 0; j < OLD_COUNT; j++)
			oldMaskList[j] = 0;						// there's no old states

		if (bitMaskList)
		{
			for (int i = 0; i < lightCount; i++)	// for all lights
			{
				if (bitMaskList[i] & LIGHT_IS_TOGGLE)	// ca 09/28/94
					bitMaskList[i] |= LIGHT_IS_ON;		// turn on the light
			}
		}
		bitMaskList[BOX_FAILURE_LIGHT] &= ~LIGHT_IS_ON;	// keep this one off
		bitMaskList[BOX_INSTR_2_LIGHT] &= ~LIGHT_IS_ON;	// keep this one off

		errcode = drawThis();						// draw them

		readyForUse = 1;	// next time drawThis() can call drawing code
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** virtual int drawGraphic() = 0;
** virtual int drawText() = 0;
** virtual int drawSpecial() = 0;
**
**		These functions draw one light.  They must be defined in the
** derived class, even if the function does nothing.
**
*************************************************************************/


/************************************************************************
** Patterns of lights
**
**		The following functions all should set up the proper lights for
** the function, then call drawThis().  The meanings of these functions
** are class-independent, though the nature of the lights in each
** class can differ.
**
*************************************************************************/


/************************************************************************
** responseOff()
**
**		Turns off all lights, excluding the banner and frame lights,
** and, internally, the background light.
**
**		If desired, can have them all turned off by passing a
** non-zero allofthem parameter.
**
*************************************************************************/

int
ResponseBoxInterior::responseOff
(
	int allofthem
)
{
	int errcode = 0;

	if (bitMaskList)
	{
		allOff();
		if (allofthem)			// force a couple more lights off
		{
			bitMaskList[BOX_BANNER_LIGHT] &= ~LIGHT_IS_ON;
			bitMaskList[BOX_FRAME_LIGHT] &= ~LIGHT_IS_ON;
		}
		errcode = drawThis();
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** lightOn()
**
**		Turns on the desired light.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the desired light, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
*************************************************************************/

int
ResponseBoxInterior::lightOn
(
	ResponseBoxLightIndex lite,
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[lite] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** readyLight()
**
**		Turns on the READY_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_READY_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
*************************************************************************/

int
ResponseBoxInterior::readyLight
(
	int offem
)
{
	int errcode = 0;

	noMoreBanner = 1;			// turn off main banner now

	if (offem)
		allOff();

	bitMaskList[BOX_READY_LIGHT] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** failureLight()
**
**		Turns on the BOX_FAILURE_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_FAILURE_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
*************************************************************************/

int
ResponseBoxInterior::failureLight
(
	int offem
)
{
	int errcode = 0;

	noMoreBanner = 0;			// turn banner on (emphasize error locus)

	if (offem)
		allOff();

	bitMaskList[BOX_FAILURE_LIGHT] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** bannerLight()
**
**		Turns on the BOX_BANNER_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_BANNER_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
*************************************************************************/

int
ResponseBoxInterior::bannerLight
(
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_BANNER_LIGHT] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** answerLight()
**
**		Turns off all lights except the BOX_ANSWER_LIGHT.
**
*************************************************************************/

int
ResponseBoxInterior::answerLight
(
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_ANSWER_LIGHT] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** intervalLight()
**
**		Turns off all lights except one of these lights:  BOX_INTERVAL_1,
** BOX_INTERVAL_2, or BOX_INTERVAL_3.  The parameter ranges from 1 to 3.
**
*************************************************************************/

int
ResponseBoxInterior::intervalLight
(
	int interval,
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_INTERVAL_1-1+interval] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** responseLight()
**
**		Turns off all lights except one of these lights:  BOX_RESPONSE_1,
** BOX_RESPONSE_2, or BOX_RESPONSE_3.  The parameter ranges from 1 to 3.
**
*************************************************************************/

int
ResponseBoxInterior::responseLight
(
	int response,
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_RESPONSE_1-1+response] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** wrongLight()
**
**		Turns off all lights except the BOX_WRONG_RESPONSE.
**
*************************************************************************/

int
ResponseBoxInterior::wrongLight
(
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_WRONG_RESPONSE] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** rightLight()
**
**		Turns off all lights except the BOX_RIGHT_RESPONSE.
**
*************************************************************************/

int
ResponseBoxInterior::rightLight
(
	int offem
)
{
	int errcode = 0;

	if (offem)
		allOff();

	bitMaskList[BOX_RIGHT_RESPONSE] |= LIGHT_IS_ON;
	errcode = drawThis();
	return errcode;
}


/************************************************************************
** feedbackLight()
**
**		Turns off all lights except one of the interval lights and
** one of the response lights.
**
*************************************************************************/

int
ResponseBoxInterior::feedbackLight
(
	int interval,
	int response
)
{
	int errcode = 0;

	errcode = intervalLight(interval, KEEP_THEM_ON);
	if (!errcode)
		errcode = responseLight(response, KEEP_THEM_ON);
	if (!errcode)
		errcode = drawThis();

	return errcode;
}


/************************************************************************
** getState()
**
**		Looks up the entity'th bit in a list of 8-bit bit-masks called
** "oldMaskList".  We go to the entity/8th byte, and mask on the
** entity%8th bit.  The value of this bit is returned.
**
**		Right now I act superstitious and don't try to do any
** 1 << 0 operations.
**
*************************************************************************/

inline unsigned
ResponseBoxInterior::getState
(
	int entity						// ranges from 0 to 256
)
{
	unsigned maskindex = entity / 8;

	entity = ((entity % 8) != 0) ? (1 << entity % 8) : 1 ;

	return((oldMaskList[maskindex] & (unsigned) entity) != 0);
}


/************************************************************************
** setState()
**
**		The inverse of getState().  Sets the proper bit of the proper
** unsigned char in "oldMaskList" to the value (0 or 1) given.
**
*************************************************************************/

inline void
ResponseBoxInterior::setState
(
	int entity,						// ranges from 0 to 256
	int value						// 0 or non-zero only
)
{
	unsigned maskindex = entity / 8;	// which oldMaskList[] entry?
	unsigned mask = 					// which bit in that entry?
		((entity % 8) != 0) ? (1 << entity % 8) : 1 ;

	if (value)
		oldMaskList[maskindex] |= mask;		// set the bit
	else
		oldMaskList[maskindex] &= ~mask;	// clear the bit
}


/************************************************************************
** virtual int drawThis()
**
**		Draws the current state of the screen.  This means redrawing
** all the lights in the interior.  To save time, it reviews the old
** state of the screen, to see what needs changing.
**
**		I'm pretty sure my bit-manipulations are NON-PORTABLE CODE!
**
*************************************************************************/

int
ResponseBoxInterior::drawThis ()
{
	int errcode = 0;

	if (bitMaskList)
	{
		if (!graphicsReady)
			errcode = 1;

		for (int i = 0; i < lightCount; i++)
		{
			/************************************************************
			** We redraw only if the new state is different from the old.
			*************************************************************/

			unsigned oldstate = getState(i);
			unsigned newstate = (bitMaskList[i] & LIGHT_IS_ON) != 0;

			if (oldstate != newstate)					// light state changed
			{
				if (bitMaskList[i] & LIGHT_IS_SPECIAL)	// light is special
				{
					if (errcode)
						cputs("drawSpecial() needs graphics\r\n");
					else
						errcode = drawSpecial(i, newstate);
				}
				else									// light not special
				{
					unsigned lpboffset =
						((bitMaskList[i] & LIGHT_PARAMETERS) >> 8) & 0x00FF;

					if (bitMaskList[i] & LIGHT_IS_TEXT)
					{
						if (errcode)
							cputs("drawText() needs graphics\r\n");
						else
							errcode = drawText(lpboffset, newstate);
					}
					else
					{
						if (errcode)
							cputs("drawGraphic() needs graphics\r\n");
						else
							errcode = drawGraphic(lpboffset, newstate);
					}
				}
				if (!errcode)
					setState(i, newstate);
			}
		}
	}
	return errcode;
}
