/************************************************************************
**
** @(#)rbox_tv.cpp		06/22/94		Chris Ahlstrom
**
** C++ version, requires Turbo Vision by Borland
**
**	Edit with tab-stops = 4.
**
**		This module interfaces C++ with the standard FACE response
** light code (in FACE_TV.LIB), and with code for handling various kinds
** of button devices.
**
**		It is intended to be a more universal way of implementing
** a response-box interior than that in respintr.cpp/h.
**
**		The procedure displays the following layout (note that, at
** present, display of the text strings is not implemented):
**
**
**		 ---------							 ---------
**		|  READY  |							| ANSWER |
**		 ---------							 ---------
**		 ---------							 ---------
**		|		  |							|		 |
**		|		  |							|		 |
**		|		  |							|		 |
**		 ---------							 ---------
**
**				 ----------			 ----------
**				|Interval 1|		|Interval 2|
**				 ----------			 ----------
**				 ----------			 ----------
**				|		   |		|		   |
**				|		   |		|		   |
**				|		   |		|		   |
**				 ----------			 ----------
**
**
**				 ----------			 ----------
**				|Response 1|		|Response 2|
**				 ----------			 ----------
**				 ----------			 ----------
**				|		   |		|		   |
**				|		   |		|		   |
**				|		   |		|		   |
**				 ----------			 ----------
**
**
**		See rbox_int.cpp/h for an explanation about the representation
** of these lights.
**
**		The following functions do not know about fonts and graphics.
** They are wholly text-based and Turbo Vision based.
**
*************************************************************************/

#define RBOX_TV_cpp

#include "rbox_tv.h"			// ResponseBoxTView class

extern "C"						// avoid name-mangling
{
	#include "math_aux.h"		// simple math like iround()
	#include "textlog.h"		// getScreenModes() [graphics code!]
}


/************************************************************************
** ResponseBoxTView constructor and destructor.
**
**		Note that, since we allocate a semi-permanent textBox, it
** might prove wise to add an assignment operator and copy constructor,
** or make them private.
**
*************************************************************************/

ResponseBoxTView::ResponseBoxTView
(
	ResponseBoxLight resplightparameters[],
	int resplightcount,
	const TRect &bounds						// size of the window
) :
	TView			(bounds),				// initialize the TView object
	respRows		(25),
	respColumns		(80),
	respPixelX		(0),
	respPixelY		(0),
	graphBox		(&resplightparameters[0]),
	textBox			(new ResponseBoxLight[lightCount]),
	intervalsUsed	(2),					// later will support 3, too
	lightCount		(resplightcount)
{
	Screen *sc = new Screen;					// allocate a struct
	if (sc)
	{
		(void) getScreenModes(sc);				// obtain many parameters
		respRows		= sc->screen_rows;		// store number of text rows
		respColumns		= sc->screen_columns;	// store number of text columns
		respPixelX		= sc->screen_gmax_x;	// store pixel depth
		respPixelY		= sc->screen_gmax_y;	// store pixel width
		delete sc;								// get rid of struct
	}
	options |= ofFramed;				// frame the box
	growMode = gfGrowHiX | gfGrowHiY;	// size same as window's

	for (int lite = 0; lite < lightCount; lite++)
	{
		if
		(
			(lite != BOX_RESPONSE_3 && lite != BOX_INTERVAL_3)
					||
			intervalsUsed == 3
		)
			lightStatus[lite] = BRESP_ON;
	}
}


ResponseBoxTView::~ResponseBoxTView ()
{
	if (textBox)
		delete [] textBox;
}


/************************************************************************
** boxAspectConvert()
**
**		Generates the proper coordinates to specify the layout
** of a response box, given the type of response box settings in use
** for the current graphics monitor.
**
**		In addition, the coordinates are scaled to fit inside a box
** of a particular size (in units of lines and characters).
**
**		This routine is called in the draw() routine, which is overridden
** here. By calling it there, the size of the lights will somewhat
** follow the size of the box, if the user resizes it.
**
*************************************************************************/

void
ResponseBoxTView::boxAspectConvert
(
	TRect *wbox						// window box into which to fit it all
)
{
	for (int i = 0; i < lightCount; i++)
	{
		lightAspectConvert(wbox, graphBox[i], textBox[i]);
	}
	//
	// Not used right now anyway, so will fix later
	//
	// textAspectConvert
	// (
	//		wbox,
	//		graphBox->failure_light,
	//		textBox->failure_light
	// );
	//
}


#if defined(__BORLANDC__)
#pragma warn -sig				// { ignore double-to-int warning
#endif

/************************************************************************
** lightAspectConvert()
**
**		Determines the coordinates of a response light relative to
** the response-box window in which it resides. All coordinates,
** whether in graphics modes or text modes, are converted to text-
** character cells.
**
**		Also copies some other fields from the source (virtual) light.
**
**		Guide to conversions.  The "screen_" values are as per
** "screen.c" and "screen.h" in the FACE library.
**
** 1. Converting virtual coordinates to characters:
**
**		a. screen_x = (virtual_x / VIRTmax) * screen_columns
**		b. screen_y = (virtual_y / VIRTmax) * screen_rows
**
** 2. Converting virtual coordinates to pixels:
**
**		a. screen_x = (virtual_x / VIRTmax) * screen_gmax_x
**		b. screen_y = (virtual_y / VIRTmax) * screen_gmax_y
**
**		This routine doesn't handle graphics modes yet, or at
** least, I haven't tested it.
**			
*************************************************************************/

void
ResponseBoxTView::lightAspectConvert
(
	TRect *wbox,				// window box into which the light goes
	ResponseBoxLight &source,		// pixel-coordinates of the light
	ResponseBoxLight &text			// text-coordinate-offsets of the light
)
{
	int va, vb;
	double maxx;
	double maxy;
	double boxx = (double) (wbox->b.x - wbox->a.x);
	double boxy = (double) (wbox->b.y - wbox->a.y);

	text.fillstyle = source.fillstyle;			// style of background
	text.color		= source.color;				// color of response light

	if (respPixelX == 0 || respPixelY == 0)
	{											// we're in text mode
		maxx = (boxx / VIRTmax);
		maxy = (boxy / VIRTmax);
	}
	else
	{											// we're in graphics mode
		maxx = (respPixelX+1) / VIRTmax / boxx;
		maxy = (respPixelY+1) / VIRTmax / boxy;
	}

	va = iround(source.x0 * maxx);
	vb = iround(source.x1 * maxx);
	if ((vb-va) <= 0)								// keep it > 0
		vb = va + 1;
	text.x0 = va;
	text.x1 = vb;

	va = iround(source.y0 * maxy);
	vb = iround(source.y1 * maxy);
	if ((vb-va) <= 0)								// keep it > 0
		vb = va + 1;
	text.y0 = va;
	text.y1 = vb;
}


/************************************************************************
** textAspectConvert()
**
**		Once the above works, fix this to suit.
**
*************************************************************************/

void
ResponseBoxTView::textAspectConvert
(
	TRect *wbox,				// window box into which the light goes
	ResponseBoxText& light,		// pixel-coordinates of the light
	ResponseBoxText& text			// text-coordinate-offsets of the light
)
{
	double maxx = (double) (respPixelX + 1);
	double maxy = (double) (respPixelY + 1);
	double boxx = (double) (wbox->b.x - wbox->a.x);
	double boxy = (double) (wbox->b.y - wbox->a.y);

	if (maxx <= 0.0 || maxy <= 0.0)
	{
		maxx = 1.0;
		maxy = 1.0;
	}

	text.x0 = iround((light.x0 / maxx) * boxx);
	text.y0 = iround((light.y0 / maxy) * boxy);
}

#if defined(__BORLANDC__)
#pragma warn .sig				// } put warning back to original state
#endif


/************************************************************************
** draw() override
*************************************************************************/

void
ResponseBoxTView::draw()
{
	uchar wcolor = (uchar) BACKGROUND_COLOR;

	TRect wbox = getExtent();			// size of the window
	wbox.grow(-1, -1);					// make it fit in window
	boxAspectConvert(&wbox);			// get new (maybe) sizes

	for (int line = 0; line < size.y; line++)	// for each line...
	{
		writeChar								// fill with colored space
		(
			0, line, (short) RCELL, wcolor, size.x
		);
	}
	for (int lite = 0; lite < lightCount; lite++)
	{
		lightDraw(lite);
	}
}


/************************************************************************
** lightDraw()
**
**		Turns on/off the desired response light.
**
**		Draws the lights according to each light's status.  This status is
** set by lightOn() or lightOff(), and the lights "wait" for the
** next order to be re-drawn all at once.
**
*************************************************************************/

void
ResponseBoxTView::lightDraw
(
	int lightnumber
)
{
	int width = textBox[lightnumber].x1 - textBox[lightnumber].x0 + 1;
	uchar lcolor;

	lcolor = lightStatus[lightnumber] ?
		textBox[lightnumber].color : BACKGROUND_COLOR;

	for											// each line in light
	(
		int i = textBox[lightnumber].y0; i <= textBox[lightnumber].y1; i++
	)
		writeChar								// fill with colored space
		(
			textBox[lightnumber].x0, i, (short) RCELL, lcolor, width
		);
}



/************************************************************************
** light()
**
**		Turns on/off the desired combinations of response lights.
** Doesn't draw the lights, just sets each status properly, then
** calls drawView() to draw the lights using draw().
**
**		NEVER CALLED!
**
*************************************************************************/

void
ResponseBoxTView::light
(
	unsigned char light_mask
)
{
	int temp_mask;

	for (int lite = 0; lite < lightCount; lite++)
	{
		temp_mask = light_mask & 1;			// get right-most bit

		if (temp_mask != 0)					// turn a light ON
			lightOn(lite);
		else
			lightOff(lite);

		light_mask >>= 1;					// shift to next bit
	}
	drawView();								// update the view
}


/************************************************************************
** lightOn()
** lightOff()
**
**		Turns on or off a single light. Since drawing of the lights
** can be redone at any time, we must always have the current status
** of lights available. Hence, these two functions are very simple...
** they merely set or reset the state flag for the light.
**
*************************************************************************/

void
ResponseBoxTView::lightOn
(
	int lightnumber
)
{
	lightStatus[lightnumber] = BRESP_ON;
}

void
ResponseBoxTView::lightOff
(
	int lightnumber
)
{
	lightStatus[lightnumber] = BRESP_OFF;
}


