/************************************************************************
**
** @(#)tmouse.c		01/07/96	Chris Ahlstrom
**
**	This module provides mouse utilities that are functionally
** partly a subset of the regular MSMOUSE routines, but have different
** names and parameter-passing conventions.
**
**	They are defined her for convenience, and to avoid using the
** whole mouse library, since Turbo Vision already provides most of
** the functionality we need.
**
*************************************************************************/

#define TINPMOUS_c

#include <dos.h>

#include "tmouse.h"


/************************************************************************
** LOCAL VERSIONS OF MOUSE ROUTINES
*************************************************************************/

#define MOUSEBIOS		0x33			/* interrupt to control mouse	*/



/************************************************************************
** mouseReset()
**
**		Implements mouse BIOS function 0
**
**		This routine initializes the mouse driver (if present) and
** returns the number of buttons (0 if no mouse driver).  It should be
** called after any change of video mode number.
**
*************************************************************************/

#define RESETMOUSE		0x00			/* resets the mouse				*/

int
mouseReset (void)
{
	union REGS mousreg;
	int buttons = 0;

	mousreg.x.ax = RESETMOUSE;

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call the mouse interrupt	*/

	if (mousreg.x.ax == 0xFFFF)
		buttons = (int) mousreg.x.bx;

	return buttons;
}


/************************************************************************
** mouseShowCursor()
**
**		Implements mouse BIOS function 1
**
**		Function instructs the mouse driver to show the mouse cursor
** and track the mouse motion. If multiple calls to m_hide_cursor
** are done, you must make multiple calls to m_show_cursor.
**
**		This routine is redundant in Turbo Vision, but we sometimes need
** control independently of Turbo Vision.
**
**	   When printing to or updating the screen while the mouse cursor is
** showing, the mouse sometimes has problems, obliterating your changes
** when it moves away after you wrote directly over the mouse cursor.
** It is usually best to hide the cursor momentarily while you update
** the screen. 
**
*************************************************************************/

#define MOUSESHOWCURSOR	0x01			/* shows the cursor				*/

void
mouseShowCursor (void)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSESHOWCURSOR;

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call the mouse interrupt	*/
}


/************************************************************************
** mouseHideCursor()
**
**		Implements mouse BIOS function 2
**
**		Function instructs the mouse driver to hide the mouse cursor
** and track the mouse motion. If multiple calls are made, then
** the caller must make multiple calls to mouseShowCursor() to get
** it back on screen.
**
**		This routine is redundant in Turbo Vision, but we sometimes need
** control independently of Turbo Vision.
**
*************************************************************************/

#define MOUSEHIDECURSOR	0x02			/* hides the cursor				*/

void
mouseHideCursor (void)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSEHIDECURSOR;

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call the mouse interrupt	*/
}


/************************************************************************
** readMouseButtons()
**
**		Based on Microsoft mouse function 3
**
**		Returns the status of the mouse buttons.
**
**	button is a bitmask; the following values apply:
**
**		   Bit   Meaning if on
**		  -----  --------------------------------------------
**			0	 left button is down (LEFT_MOUSEBUTTON)
**			1	 right button is down (RIGHT_MOUSEBUTTON)
**			2	 center button is down (CENTER_MOUSEBUTTON)
**			3 to 15 are reserved
**
**		I created this routine because I'm not satisfied with the
** behavior of mouseEvent().
**
*************************************************************************/

#define READMOUSE		0x03			/* function to read mousy stuff	*/

int
readMouseButtons (void)
{
	union REGS mousreg;

	mousreg.x.ax = READMOUSE;
	mousreg.x.bx = 0;						/* set for no buttons		*/

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call the mouse interrupt	*/
	return mousreg.x.bx;
}


/************************************************************************
** readMouse()
**
** Based on Microsoft mouse function 3
**
**		The same as readMouseButtons().  In addition, it returns the
** current coordinates of the mouse.
**
*************************************************************************/

int
readMouse
(
	int *x,
	int *y
)
{
	union REGS mousreg;

	mousreg.x.ax = READMOUSE;

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call the mouse interrupt	*/

	*x = mousreg.x.cx;
	*y = mousreg.x.dx;

	return mousreg.x.bx;
}


/************************************************************************
** setMousePosition()
**
**		Based on Microsoft mouse function 4, but used in text-mode
** for certain TINP*.CPP/H module functions.  For graphics mode usage,
** see setMousePixel().
**
**		Places the mouse at position (x,y).  The position is adjusted,
** if necessary, to lie within the confines specified in any previous
** calls to functions 7 and 8 (setmousehoriz() and setmousevert()).
**
**		One problem is that the vertical dimension correction factor
** is text-mode dependent.
**
*************************************************************************/

#define MOUSEPOSITION	0x04			/* function to move the mouse	*/

void
setMousePosition
(
	int x,
	int y
)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSEPOSITION;
	mousreg.x.cx = x * MOUSE_CHAR_WIDTH;	/* convert to mouse units	*/
	mousreg.x.dx = y * MOUSE_CHAR_HEIGHT;	/* convert to mouse units	*/

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/
}


/************************************************************************
** setMousePixel()
**
**		Implements Microsoft mouse function 4.  Used in graphics mode
** only; see setMousePosition() for the text-mode version.
**
**		Places the mouse at position (x,y).  The position is adjusted,
** if necessary, to lie within the confines specified in any previous
** calls to functions 7 and 8 (setmousehoriz() and setmousevert()).
**
*************************************************************************/

void
setMousePixel
(
	int x,
	int y
)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSEPOSITION;
	mousreg.x.cx = x;
	mousreg.x.dx = y;

	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/
}


/************************************************************************
** mousePresses()
**
**		Based on Microsoft mouse function 5
**
**		Returns the number of button presses since the last call.  The
** (x,y) position of the last button press is also returned.
**
**	NOTE THE UNUSED PARAMETER... it indicates we are not handling this
**  call PROPERLY!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
**
*************************************************************************/

#define MOUSEPRESSES	0x05			/* obtain button Presses		*/

int
mousePresses
(
 /*	int bs, */				/* button selection						*/
	int *ix,			/* x coordinate of last button press	*/
	int *iy				/* y coordinate of last button press	*/
)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSEPRESSES;			/* do this function			*/
	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/

	/********************************************************************
	** mousereg.x.ax contains status of buttons in same format as for
	** mousereg.x.bx in readMouseButtons() [mouse BIOS function 3].
	*********************************************************************/

	*ix = mousreg.x.cx;						/* return horizontal coord.	*/
	*iy = mousreg.x.dx;						/* return vertical coord.	*/

	return mousreg.x.bx;					/* return presses count		*/
}


/************************************************************************
** readYmickey()
**
**		Based on Microsoft mouse function 11
**
**		A global function to return the number of vertical mickeys
** the mouse moved since the last call to readYmickey.
**
**		I copped this routine from the MSMOUSE library and tailored
** it to our limited usage.
**
**		Reads the motion of the mouse (its displacement since the
** last call), in units of mickeys.  Discards the x displacement, and
** returns the y displacement.  A positive number indicates travel
** downwards.  One mickey is about 1/200th inch of mouse movement.
**
**		Note that even if the mouse cursor is at the edge of its
** area, and stuck there because it can go no farther, this function
** still returns non-zero mickey values as long as the user continues
** to move the mouse.
**
*************************************************************************/

#define READMICKEY		0x0B			/* function to read mickeys		*/

int
readYmickey (void)
{
	union REGS mousreg;

	mousreg.x.ax = READMICKEY;				/* do this function			*/
	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/
	return mousreg.x.dx;					/* return vertical movement	*/
}


/************************************************************************
** monoMouseCursor() and mainMouseCursor() are defined in tmtomono.asm.
*************************************************************************/


#ifdef USE_THIS STUFF

	We'll add these if the need arises.

/************************************************************************
** mouseReleases()
**
**		Based on Microsoft mouse function 6
**
**		Returns the number of button releases since the last call.  The
** (x,y) position of the last button press is also returned.
**
*************************************************************************/

#define MOUSERELEASES	0x06			/* obtain button releases		*/

int
mouseReleases
(
	int bs,				/* button selection						*/
	int *ix,			/* x coordinate of last button press	*/
	int *iy				/* y coordinate of last button press	*/
)
{
	union REGS mousreg;

	mousreg.x.ax = MOUSERELEASES;			/* do this function			*/
	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/
	*ix = mousreg.x.cx;						/* return horizontal coord.	*/
	*iy = mousreg.x.dx;						/* return vertical coord.	*/

	return mousreg.x.bx;					/* return release count		*/
}


/************************************************************************
** mouseRegion()
**
**		Based on Microsoft mouse functions 7 and 8.
**
**		Restricts the mouse motion to a given rectangular region
** on the screen.
**
*************************************************************************/

#define MOUSEREGION_H	0x07			/* set horizontal limits		*/
#define MOUSEREGION_V	0x08			/* set vertical limits			*/

void
mouseRegion
(
	int hmin,
	int hmax,
	int vmin,
	int vmax
)
{
	union REGS mousreg;

	if ((hmin >= 0) && (hmax >= 0))
	{
		mousreg.x.ax = MOUSEREGION_H;			/* do this function		*/
		mousreg.x.cx = hmin;
		mousreg.x.dx = hmax;
		int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt	*/
	}
	if ((vmin >= 0) && (vmax >= 0))
	{
		mousreg.x.ax = MOUSEREGION_V;			/* do this function		*/
		mousreg.x.cx = vmin;
		mousreg.x.dx = vmax;
		int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt	*/
	}
}


/************************************************************************
** mouseCursor()
**
**		Based on Microsoft mouse function 9
**
**		Changes the shape of the mouse cursor in graphics mode.  You
** cannot change the shape of the text mode cursor.
**
*************************************************************************/

#define MOUSECURSOR		0x09			/* set shape of the cursor		*/

void
mouseCursor
(
	int *graphics						/* what the HELL is this?		*/
)
	union REGS mousreg;

	mousreg.x.ax = MOUSECURSOR;				/* do this function			*/
	mousreg.x.bx = 0xFFFF;
	mousreg.x.cx = 0xFFFF;
	mousreg.x.dx = (int) graphics;
	int86(MOUSEBIOS, &mousreg, &mousreg);	/* call mouse interrupt		*/
}


#endif
