/************************************************************************
**
** @(#)umessage.cpp	03/28/94	Chris Ahlstrom
**
**	C++ Borland/Microsoft
**
**	Defines the default routines for the base class UserMessage.
** This class implements a pretty simple and straightforward way
** of handling user- and error-messages.
**
**	This class can be used to derive other classes, and from it the
** derived class inherits a small set of public and private functions
** and private data members to support the providing of error codes
** and pointers to internal messages to the outside.
**
**	This class knows nothing about displaying errors.  It is up
** to the derived object to get the pointer to the message and
** pass it along for display.
**
*************************************************************************/

#define UMESSAGE_cpp


#include <stdio.h>			// for sprintf()

#include "umessage.h"			// UserMessage class declaration


/************************************************************************
** UserMessage constructors and destructor
**
**	Note that "msglist" is a constant pointer to a constant
** list of pointers to error-messages strings that are each constant.
**
**	We also provide two constructors... one for setting up and
** external list of messages, and one for simply returning a
** string to display an error code.
**
**	Later on, I would like to find some way of concatenating
** message lists and yet properly handling the error codes that
** correspond to them.
**
*************************************************************************/

UserMessage::UserMessage		// constructor
(
    const char * const * const msglist,	// list of messages
    int msgcount			// number of them provided
) :
    szMessage		((char *)0),	// no message to start with
    szMessageList	(msglist),	// a list of messages
    szMessageCount	(msgcount),	// the number of them
    errorCode		(0)		// no errors to start with
{
}


UserMessage::UserMessage ()		// default constructor
  :
    szMessage		((char *)0),	// no message to start with
    szMessageList	((char *)0),	// a list of messages
    szMessageCount	(0),		// the number of them
    errorCode		(0)		// no errors to start with
{
}


UserMessage::~UserMessage ()
{
}


/************************************************************************
** hookMessages()
**
**	An alternate way of hooking in the messages, useful when
** the UserMessage object is a class member (as opposed to a class base).
**
*************************************************************************/

void
UserMessage::hookMessages
(
    const char * const * const msglist,	// list of messages
    int msgcount			// number of them provided
)
{
    szMessage		= 0;		// zero the message pointer
    szMessageList	= msglist;	// a list of messages
    szMessageCount	= msgcount;	// the number of them
    errorCode		= 0;		// zero the error code
}


/************************************************************************
** isError()
**
**	Returns a 1 if the errorCode is non-zero.
**
**	Should not need overriding, unless some quantum increase
** in robustness is need for a particular object.  I don't even
** know if you can override an inline function.
**
**	This function is meant as a much faster alternative to calling
** getError() to see if there is an error message pending.
**
**	Use isError() if you know that there's no built-in error return
** code for a particular function.
**
**	See the header file umessage.h for the definition.
**
*************************************************************************/



/************************************************************************
** getErrorCode()
**
**	Returns the value of errorCode.  I'm not going to let derived
** objects have direct access to errorCode.  Why?  I want to practise
** using "protected" vs. "private" in inheritance.
**
**	It is an inline function defined in umessage.h.
**
**
** Message()
**
**	Simply returns a pointer to the latest message.  If there
** is no message pending, a NULL is returned.
**
**	This routine should not need to be overridden.
**
**
** errorMessage()
**
**	Simply returns a pointer to the latest error message.  If there
** is no error pending, a NULL is returned.
**
**	This routine should not need to be overridden.
**
*************************************************************************/

const char *
UserMessage::Message ()	
{
    return szMessage;
}


const char *
UserMessage::errorMessage ()	
{
    return errorCode ? szMessage : 0 ;
}


/************************************************************************
** getMessageEntry()
**
**	Returns a pointer to one of the messages in the error message
** list, if it exists.  This routine is meant to be an internal function.
**
**	This routine should not need to be overridden.  It's also
** a protected function, so it will be private in the derived class,
** since this function is meant only for housekeeping.
**
*************************************************************************/

const char *
UserMessage::getMessageEntry		// get pointer to message entry
(
    int index
)
{
    return (index < szMessageCount && index >= 0) ?
	szMessageList[index] : 0 ;
}


/************************************************************************
** setErrorCode()
**
**	All routines that perform a function that can err should
** call this routine to set both errorCode and szMessage, even if there
** is no error.
**
**	This routine should not need to be overridden.  However,
** it should be overridden when the source of messages is not the
** szMessageList.  See xbpa4.cpp for a good example.
**
*************************************************************************/

#define DEFAULTMESSAGEFORMAT	"?User error %d; see source."

static char defaultMessage[82];		// for assembling basic message

void
UserMessage::setErrorCode
(
    int index
)
{
    if (szMessageCount <= 0)
    {
	if (index != 0)
	{
	    sprintf(defaultMessage, DEFAULTMESSAGEFORMAT, index);
	    szMessage = defaultMessage;
	}
    }
    else
    {
	if (index != 0)
	{
	    if (index < szMessageCount)
		szMessage = szMessageList[index]; // hopefully, it's not null
	    else
		szMessage = (char *) 0;		  // no list for this device
	}
	else
	    szMessage = (char *) 0;
    }
    errorCode = index;
}

